rm(list = ls())
library(seewave)
library(tuneR)
library(forecast)
library(signal)

setwd('')

H1 <- readWave("TR1_H1.wav")
H1 <- H1@left
fs <- 48000

#Generate butterworth filter coefficients

b <- butter(3,c(1500,20000)/(fs/2))$b
a <- butter(3,c(1500,20000)/(fs/2))$a

H1 <- filtfilt(b,a,H1)

#Detach signal to avoid masking steps later on
detach('package:signal')

#Locations (in samples) of snapping shrimp snaps
snap_time <- c(round(14.9775 * fs)+13, round(81.82325 * fs)-2, round(2.38355 * fs)-5,round(11.4805*fs))

#Isolate 200 point sections where snaps occur in waveform, find the absolute value of 
#Hilbert transform, and normalize each wave. Output should be four columns with different
#waveforms.
mat.env = matrix(0,ncol = 4,nrow = 199)
for (i in 1:length(snap_time)){
  wav <- H1[(snap_time[i]-10):(snap_time[i]+199+10)]
  h_tform <- abs(hilbert(wav,fs))
  h_tform <- h_tform[11:(length(h_tform)-11)]/(max(h_tform))
  wav <- wav[11:(length(wav)-11)]
  mat.env[,i] <- h_tform
}

#Find mean down columns and smooth (5 = span of moving average). 
#Replace NA values with 0

avg_snap <- rowMeans(mat.env)

snap_sm <- ma(avg_snap,5)

snap_sm[is.na(snap_sm)] <- 0

# plot(snap_sm[1:35],type = "l")
# points(snap_sm[36:65],type = "l",col = "green")
# points(snap_sm[66:120],type = "l",col = "blue")

#Use polynomial smoothing on waves and find fitted values
p1 <- lm(snap_sm[1:35]~poly((1:35),5))$fitted.values
p2 <- lm(snap_sm[36:65]~poly((36:65),5))$fitted.values
p3 <- lm(snap_sm[66:120]~poly((66:120),5))$fitted.values

snap_sm <- list(p1,p2,p3)
for (i in 1:length(snap_sm)){
  snap_sm[[i]] <- snap_sm[[i]]- tail(snap_sm[[3]],n = 1L)
}

#Build kernel and normalize
kern <- c(snap_sm[[1]],snap_sm[[2]],snap_sm[[3]])
kern[kern<0] <- 0
kern <- kern/max(kern)
kern <- ma(kern,3)
kern[is.na(kern)] <- 0
kern <- kern/max(kern)

#klength = kernel length, pk = peak location (in samples)
klength <- length(kern)
pk <- which.max(kern)

#Check out how your kernel matches with one of the waveforms you started with!
plot(mat.env[,1],type = "l",las = 1, ylab = "Normalized Amplitude",xlab = 'Samples',lwd = 2)
points(kern,type = "l",col = "red",lwd = 2)
legend(125,0.9,legend = c("Kernel","Sample Waveform"),col = c("red","black"),lty=1,bty = "n",lwd = 2)

kern_t50 <- kern
#Save kernel for use in detection algorithm
setwd("C:/")
write.table(kern_t50, file = "kernel_t50.txt",quote = TRUE, sep = ",", na = "0",
            dec = ".") 
